@@ -37,6 +37,7 @@
 /* #define G_MAIN_POLL_DEBUG */
 
 #include "glib.h"
+#include "gthreadinit.h"
 #include <sys/types.h>
 #include <time.h>
 #ifdef HAVE_SYS_TIME_H
@@ -153,7 +154,7 @@ struct _GTimeoutSource
 {
   GSource     source;
   GTimeVal    expiration;
-  gint        interval;
+  guint       interval;
 };
 
 struct _GPollRec
@@ -174,6 +175,8 @@ struct _GPollRec
 #endif
 
 #define SOURCE_DESTROYED(source) (((source)->flags & G_HOOK_FLAG_ACTIVE) == 0)
+#define SOURCE_BLOCKED(source) (((source)->flags & G_HOOK_FLAG_IN_CALL) != 0 && \
+		                ((source)->flags & G_SOURCE_CAN_RECURSE) == 0)
 
 #define SOURCE_UNREF(source, context)                       \
    G_STMT_START {                                           \
@@ -219,6 +222,7 @@ static gboolean g_idle_dispatch    (GSource     *source,
 
 G_LOCK_DEFINE_STATIC (main_loop);
 static GMainContext *default_main_context;
+static GSList *main_contexts_without_pipe = NULL;
 
 #if defined(G_PLATFORM_WIN32) && defined(__GNUC__)
 __declspec(dllexport)
@@ -267,17 +271,19 @@ g_poll (GPollFD *fds,
   for (f = fds; f < &fds[nfds]; ++f)
     if (f->fd >= 0)
       {
-	if (f->events & G_IO_IN)
+	if (f->fd == G_WIN32_MSG_HANDLE)
+	  poll_msgs = TRUE;
+	else if (nhandles == MAXIMUM_WAIT_OBJECTS)
+	  {
+	    g_warning (G_STRLOC ": Too many handles to wait for!\n");
+	    break;
+	  }
+	else
 	  {
-	    if (f->fd == G_WIN32_MSG_HANDLE)
-	      poll_msgs = TRUE;
-	    else
-	      {
 #ifdef G_MAIN_POLL_DEBUG
-		g_print ("g_poll: waiting for %#x\n", f->fd);
+	    g_print ("g_poll: waiting for %#x\n", f->fd);
 #endif
-		handles[nhandles++] = (HANDLE) f->fd;
-	      }
+	    handles[nhandles++] = (HANDLE) f->fd;
 	  }
       }
 
@@ -308,7 +314,11 @@ g_poll (GPollFD *fds,
 		  g_print ("WaitMessage\n");
 #endif
 		  if (!WaitMessage ())
-		    g_warning (G_STRLOC ": WaitMessage() failed");
+		    {
+		      gchar *emsg = g_win32_error_message (GetLastError ());
+		      g_warning (G_STRLOC ": WaitMessage() failed: %s", emsg);
+		      g_free (emsg);
+		    }
 		  ready = WAIT_OBJECT_0 + nhandles;
 		}
 	      else if (timeout == 0)
@@ -326,7 +336,12 @@ g_poll (GPollFD *fds,
 		   */
 		  timer = SetTimer (NULL, 0, timeout, NULL);
 		  if (timer == 0)
-		    g_warning (G_STRLOC ": SetTimer() failed");
+		    {
+		      gchar *emsg = g_win32_error_message (GetLastError ());
+		      g_warning (G_STRLOC ": SetTimer() failed: %s", emsg);
+		      g_free (emsg);
+		      ready = WAIT_TIMEOUT;
+		    }
 		  else
 		    {
 #ifdef G_MAIN_POLL_DEBUG
@@ -357,7 +372,11 @@ g_poll (GPollFD *fds,
 						 timeout, QS_ALLINPUT);
 
 	      if (ready == WAIT_FAILED)
-		g_warning (G_STRLOC ": MsgWaitForMultipleObjects() failed");
+		{
+		  gchar *emsg = g_win32_error_message (GetLastError ());
+		  g_warning (G_STRLOC ": MsgWaitForMultipleObjects() failed: %s", emsg);
+		  g_free (emsg);
+		}
 	    }
 	}
     }
@@ -376,11 +395,15 @@ g_poll (GPollFD *fds,
 #endif
       ready = WaitForMultipleObjects (nhandles, handles, FALSE, timeout);
       if (ready == WAIT_FAILED)
-	g_warning (G_STRLOC ": WaitForMultipleObjects() failed");
+	{
+	  gchar *emsg = g_win32_error_message (GetLastError ());
+	  g_warning (G_STRLOC ": WaitForMultipleObjects() failed: %s", emsg);
+	  g_free (emsg);
+	}
     }
 
 #ifdef G_MAIN_POLL_DEBUG
-  g_print ("wait returns %d%s\n",
+  g_print ("wait returns %ld%s\n",
 	   ready,
 	   (ready == WAIT_FAILED ? " (WAIT_FAILED)" :
 	    (ready == WAIT_TIMEOUT ? " (WAIT_TIMEOUT)" :
@@ -407,10 +430,13 @@ g_poll (GPollFD *fds,
   else if (ready >= WAIT_OBJECT_0 && ready < WAIT_OBJECT_0 + nhandles)
     for (f = fds; f < &fds[nfds]; ++f)
       {
-	if ((f->events & G_IO_IN)
+	if ((f->events & (G_IO_IN | G_IO_OUT))
 	    && f->fd == (gint) handles[ready - WAIT_OBJECT_0])
 	  {
-	    f->revents |= G_IO_IN;
+	    if (f->events & G_IO_IN)
+	      f->revents |= G_IO_IN;
+	    else
+	      f->revents |= G_IO_OUT;
 #ifdef G_MAIN_POLL_DEBUG
 	    g_print ("g_poll: got event %#x\n", f->fd);
 #endif
@@ -527,6 +553,26 @@ g_main_context_ref (GMainContext *context)
   UNLOCK_CONTEXT (context);
 }
 
+/* If DISABLE_MEM_POOLS is defined, then freeing the
+ * mem chunk won't free the records, so we have to
+ * do it manually. The conditionalization here is
+ * an optimization; g_mem_chunk_free() is a no-op
+ * when DISABLE_MEM_POOLS is set.
+ */
+#ifdef DISABLE_MEM_POOLS
+static void
+poll_rec_list_free (GMainContext *context,
+		    GPollRec     *list)
+{
+  while (list)
+    {
+      GPollRec *tmp_rec = list;
+      list = list->next;
+      g_chunk_free (tmp_rec, context->poll_chunk);
+    }
+}
+#endif /* DISABLE_MEM_POOLS */
+
 static void
 g_main_context_unref_and_unlock (GMainContext *context)
 {
@@ -555,8 +601,14 @@ g_main_context_unref_and_unlock (GMainContext *context)
 
   g_ptr_array_free (context->pending_dispatches, TRUE);
   g_free (context->cached_poll_array);
+
+#ifdef DISABLE_MEM_POLLS
+  poll_rec_list_free (context, context->poll_records);
+  poll_rec_list_free (context, context->poll_free_list);
+#endif /* DISABLE_MEM_POOLS */
   
-  g_mem_chunk_destroy (context->poll_chunk);
+  if (context->poll_chunk) 
+    g_mem_chunk_destroy (context->poll_chunk);
 
 #ifdef G_THREADS_ENABLED
   if (g_thread_supported())
@@ -567,7 +619,10 @@ g_main_context_unref_and_unlock (GMainContext *context)
 #else
       CloseHandle (context->wake_up_semaphore);
 #endif
-    }
+    } 
+  else
+    main_contexts_without_pipe = g_slist_remove (main_contexts_without_pipe, 
+						 context);
 #endif
   
   g_free (context);
@@ -590,6 +645,45 @@ g_main_context_unref (GMainContext *context)
   g_main_context_unref_and_unlock (context);
 }
 
+#ifdef G_THREADS_ENABLED
+static void 
+g_main_context_init_pipe (GMainContext *context)
+{
+# ifndef G_OS_WIN32
+  if (pipe (context->wake_up_pipe) < 0)
+    g_error ("Cannot create pipe main loop wake-up: %s\n",
+	     g_strerror (errno));
+  
+  context->wake_up_rec.fd = context->wake_up_pipe[0];
+  context->wake_up_rec.events = G_IO_IN;
+# else
+  context->wake_up_semaphore = CreateSemaphore (NULL, 0, 100, NULL);
+  if (context->wake_up_semaphore == NULL)
+    g_error ("Cannot create wake-up semaphore: %s",
+	     g_win32_error_message (GetLastError ()));
+  context->wake_up_rec.fd = (gint) context->wake_up_semaphore;
+  context->wake_up_rec.events = G_IO_IN;
+#  ifdef G_MAIN_POLL_DEBUG
+  g_print ("wake-up semaphore: %#x\n", (guint) context->wake_up_semaphore);
+#  endif
+# endif
+  g_main_context_add_poll_unlocked (context, 0, &context->wake_up_rec);
+}
+
+void
+_g_main_thread_init ()
+{
+  GSList *curr = main_contexts_without_pipe;
+  while (curr)
+    {
+      g_main_context_init_pipe ((GMainContext *)curr->data);
+      curr = curr->next;
+    }
+  g_slist_free (main_contexts_without_pipe);
+  main_contexts_without_pipe = NULL;  
+}
+#endif /* G_THREADS_ENABLED */
+
 /**
  * g_main_context_new:
  * 
@@ -611,47 +705,29 @@ g_main_context_new ()
       
   context->ref_count = 1;
 
-      context->next_id = 1;
-      
-      context->source_list = NULL;
-
+  context->next_id = 1;
+  
+  context->source_list = NULL;
+  
 #if HAVE_POLL
-      context->poll_func = (GPollFunc)poll;
+  context->poll_func = (GPollFunc)poll;
 #else
-      context->poll_func = g_poll;
+  context->poll_func = g_poll;
 #endif
-
-      context->cached_poll_array = NULL;
-      context->cached_poll_array_size = 0;
-      
-      context->pending_dispatches = g_ptr_array_new ();
-      
-      context->time_is_current = FALSE;
-
+  
+  context->cached_poll_array = NULL;
+  context->cached_poll_array_size = 0;
+  
+  context->pending_dispatches = g_ptr_array_new ();
+  
+  context->time_is_current = FALSE;
+  
 #ifdef G_THREADS_ENABLED
-      if (g_thread_supported ())
-	{
-#ifndef G_OS_WIN32
-	  if (pipe (context->wake_up_pipe) < 0)
-	    g_error ("Cannot create pipe main loop wake-up: %s\n",
-		     g_strerror (errno));
-	  
-	  context->wake_up_rec.fd = context->wake_up_pipe[0];
-	  context->wake_up_rec.events = G_IO_IN;
-	  g_main_context_add_poll_unlocked (context, 0, &context->wake_up_rec);
-#else
-	  context->wake_up_semaphore = CreateSemaphore (NULL, 0, 100, NULL);
-	  if (context->wake_up_semaphore == NULL)
-	    g_error ("Cannot create wake-up semaphore: %s",
-		     g_win32_error_message (GetLastError ()));
-	  context->wake_up_rec.fd = (gint) context->wake_up_semaphore;
-	  context->wake_up_rec.events = G_IO_IN;
-#ifdef G_MAIN_POLL_DEBUG
-	  g_print ("wake-up semaphore: %#x\n", (guint) context->wake_up_semaphore);
-#endif
-	  g_main_context_add_poll_unlocked (context, 0, &context->wake_up_rec);
-#endif
-	}
+  if (g_thread_supported ())
+    g_main_context_init_pipe (context);
+  else
+    main_contexts_without_pipe = g_slist_prepend (main_contexts_without_pipe, 
+						  context);
 #endif
 
   return context;
@@ -695,7 +771,7 @@ g_main_context_default (void)
  * <literal>sizeof (GSource)</literal>.
  * 
  * The source will not initially be associated with any #GMainContext
- * and must be added to one with g_source_add() before it will be
+ * and must be added to one with g_source_attach() before it will be
  * executed.
  * 
  * Return value: the newly-created #GSource.
@@ -844,14 +920,17 @@ g_source_destroy_internal (GSource      *source,
 	  old_cb_funcs->unref (old_cb_data);
 	  LOCK_CONTEXT (context);
 	}
-      
-      tmp_list = source->poll_fds;
-      while (tmp_list)
+
+      if (!SOURCE_BLOCKED (source))
 	{
-	  g_main_context_remove_poll_unlocked (context, tmp_list->data);
-	  tmp_list = tmp_list->next;
+	  tmp_list = source->poll_fds;
+	  while (tmp_list)
+	    {
+	      g_main_context_remove_poll_unlocked (context, tmp_list->data);
+	      tmp_list = tmp_list->next;
+	    }
 	}
-      
+	  
       g_source_unref_internal (source, context, TRUE);
     }
 
@@ -957,7 +1036,8 @@ g_source_add_poll (GSource *source,
 
   if (context)
     {
-      g_main_context_add_poll_unlocked (context, source->priority, fd);
+      if (!SOURCE_BLOCKED (source))
+	g_main_context_add_poll_unlocked (context, source->priority, fd);
       UNLOCK_CONTEXT (context);
     }
 }
@@ -965,7 +1045,7 @@ g_source_add_poll (GSource *source,
 /**
  * g_source_remove_poll:
  * @source:a #GSource 
- * @fd: a #GPollFD structure previously passed to g_source_poll().
+ * @fd: a #GPollFD structure previously passed to g_source_add_poll().
  * 
  * Removes a file descriptor from the set of file descriptors polled for
  * this source. 
@@ -989,7 +1069,8 @@ g_source_remove_poll (GSource *source,
 
   if (context)
     {
-      g_main_context_remove_poll_unlocked (context, fd);
+      if (!SOURCE_BLOCKED (source))
+	g_main_context_remove_poll_unlocked (context, fd);
       UNLOCK_CONTEXT (context);
     }
 }
@@ -1002,8 +1083,8 @@ g_source_remove_poll (GSource *source,
  *                  and getting the callback and data
  * 
  * Sets the callback function storing the data as a refcounted callback
- * "object". This is used to implement g_source_set_callback_closure()
- * and internally. Note that calling g_source_set_callback_indirect() assumes
+ * "object". This is used internally. Note that calling 
+ * g_source_set_callback_indirect() assumes
  * an initial reference count on @callback_data, and thus
  * @callback_funcs->unref will eventually be called once more
  * than @callback_funcs->ref.
@@ -1086,7 +1167,15 @@ static GSourceCallbackFuncs g_source_callback_funcs = {
  * @data: the data to pass to callback function
  * @notify: a function to call when @data is no longer in use, or %NULL.
  * 
- * Sets the callback function for a source.
+ * Sets the callback function for a source. The callback for a source is
+ * called from the source's dispatch function.
+ *
+ * The exact type of @func depends on the type of source; ie. you
+ * should not count on @func being called with @data as its first
+ * parameter.
+ * 
+ * Typically, you won't use this function. Instead use functions specific
+ * to the type of source you are using.
  **/
 void
 g_source_set_callback (GSource        *source,
@@ -1135,16 +1224,22 @@ g_source_set_priority (GSource  *source,
 
   if (context)
     {
-      source->next = NULL;
-      source->prev = NULL;
-      
-      tmp_list = source->poll_fds;
-      while (tmp_list)
+      /* Remove the source from the context's source and then
+       * add it back so it is sorted in the correct plcae
+       */
+      g_source_list_remove (source, source->context);
+      g_source_list_add (source, source->context);
+
+      if (!SOURCE_BLOCKED (source))
 	{
-	  g_main_context_remove_poll_unlocked (context, tmp_list->data);
-	  g_main_context_add_poll_unlocked (context, priority, tmp_list->data);
-      
-	  tmp_list = tmp_list->next;
+	  tmp_list = source->poll_fds;
+	  while (tmp_list)
+	    {
+	      g_main_context_remove_poll_unlocked (context, tmp_list->data);
+	      g_main_context_add_poll_unlocked (context, priority, tmp_list->data);
+	      
+	      tmp_list = tmp_list->next;
+	    }
 	}
       
       UNLOCK_CONTEXT (source->context);
@@ -1330,7 +1425,7 @@ g_main_context_find_source_by_id (GMainContext *context,
 {
   GSource *source;
   
-  g_return_val_if_fail (source_id > 0, FALSE);
+  g_return_val_if_fail (source_id > 0, NULL);
 
   if (context == NULL)
     context = g_main_context_default ();
@@ -1370,7 +1465,7 @@ g_main_context_find_source_by_funcs_user_data (GMainContext *context,
 {
   GSource *source;
   
-  g_return_val_if_fail (funcs != NULL, FALSE);
+  g_return_val_if_fail (funcs != NULL, NULL);
 
   if (context == NULL)
     context = g_main_context_default ();
@@ -1530,7 +1625,7 @@ g_source_remove_by_funcs_user_data (GSourceFuncs *funcs,
  * g_get_current_time:
  * @result: #GTimeVal structure in which to store current time.
  * 
- * Equivalent to the UNIX <function>gettimeofday()</function> function, but portable.
+ * Equivalent to the UNIX gettimeofday() function, but portable.
  **/
 void
 g_get_current_time (GTimeVal *result)
@@ -1571,6 +1666,43 @@ g_get_current_time (GTimeVal *result)
 
 /* Running the main loop */
 
+/* Temporarily remove all this source's file descriptors from the
+ * poll(), so that if data comes available for one of the file descriptors
+ * we don't continually spin in the poll()
+ */
+/* HOLDS: source->context's lock */
+void
+block_source (GSource *source)
+{
+  GSList *tmp_list;
+
+  g_return_if_fail (!SOURCE_BLOCKED (source));
+
+  tmp_list = source->poll_fds;
+  while (tmp_list)
+    {
+      g_main_context_remove_poll_unlocked (source->context, tmp_list->data);
+      tmp_list = tmp_list->next;
+    }
+}
+
+/* HOLDS: source->context's lock */
+void
+unblock_source (GSource *source)
+{
+  GSList *tmp_list;
+  
+  g_return_if_fail (!SOURCE_BLOCKED (source)); /* Source already unblocked */
+  g_return_if_fail (!SOURCE_DESTROYED (source));
+  
+  tmp_list = source->poll_fds;
+  while (tmp_list)
+    {
+      g_main_context_add_poll_unlocked (source->context, source->priority, tmp_list->data);
+      tmp_list = tmp_list->next;
+    }
+}
+
 /* HOLDS: context's lock */
 static void
 g_main_dispatch (GMainContext *context)
@@ -1606,6 +1738,9 @@ g_main_dispatch (GMainContext *context)
 	  if (cb_funcs)
 	    cb_funcs->ref (cb_data);
 	  
+	  if ((source->flags & G_SOURCE_CAN_RECURSE) == 0)
+	    block_source (source);
+	  
 	  was_in_call = source->flags & G_HOOK_FLAG_IN_CALL;
 	  source->flags |= G_HOOK_FLAG_IN_CALL;
 
@@ -1625,6 +1760,10 @@ g_main_dispatch (GMainContext *context)
 	 if (!was_in_call)
 	    source->flags &= ~G_HOOK_FLAG_IN_CALL;
 
+	  if ((source->flags & G_SOURCE_CAN_RECURSE) == 0 &&
+	      !SOURCE_DESTROYED (source))
+	    unblock_source (source);
+	  
 	  /* Note: this depends on the fact that we can't switch
 	   * sources from one main context to another
 	   */
@@ -1913,7 +2052,7 @@ g_main_context_prepare (GMainContext *context,
 	  SOURCE_UNREF (source, context);
 	  break;
 	}
-      if ((source->flags & G_HOOK_FLAG_IN_CALL) && !(source->flags & G_SOURCE_CAN_RECURSE))
+      if (SOURCE_BLOCKED (source))
 	goto next;
 
       if (!(source->flags & G_SOURCE_READY))
@@ -1966,7 +2105,7 @@ g_main_context_prepare (GMainContext *context,
  * g_main_context_query:
  * @context: a #GMainContext
  * @max_priority: maximum priority source to check
- * @timeout: location to store timeout to be used in polling
+ * @timeout_: location to store timeout to be used in polling
  * @fds: location to store #GPollFD records that need to be polled.
  * @n_fds: length of @fds.
  * 
@@ -2101,7 +2240,7 @@ g_main_context_check (GMainContext *context,
 	  SOURCE_UNREF (source, context);
 	  break;
 	}
-      if ((source->flags & G_HOOK_FLAG_IN_CALL) && !(source->flags & G_SOURCE_CAN_RECURSE))
+      if (SOURCE_BLOCKED (source))
 	goto next;
 
       if (!(source->flags & G_SOURCE_READY))
@@ -2429,7 +2568,7 @@ g_main_loop_run (GMainLoop *loop)
       if (!loop->context->cond)
 	loop->context->cond = g_cond_new ();
           
-      while (loop->is_running || !got_ownership)
+      while (loop->is_running && !got_ownership)
 	got_ownership = g_main_context_wait (loop->context,
 					     loop->context->cond,
 					     g_static_mutex_get_mutex (&loop->context->mutex));
@@ -2558,8 +2697,14 @@ g_main_context_poll (GMainContext *context,
       
       UNLOCK_CONTEXT (context);
       if ((*poll_func) (fds, n_fds, timeout) < 0 && errno != EINTR)
-	g_warning ("poll(2) failed due to: %s.",
-		   g_strerror (errno));
+	{
+#ifndef G_OS_WIN32
+	  g_warning ("poll(2) failed due to: %s.",
+		     g_strerror (errno));
+#else
+	  /* If g_poll () returns -1, it has already called g_warning() */
+#endif
+	}
       
 #ifdef	G_MAIN_POLL_DEBUG
       LOCK_CONTEXT (context);
@@ -2784,9 +2929,9 @@ g_source_get_current_time (GSource  *source,
  * @func: the function to call to poll all file descriptors
  * 
  * Sets the function to use to handle polling of file descriptors. It
- * will be used instead of the <function>poll()</function> system call 
+ * will be used instead of the poll() system call 
  * (or GLib's replacement function, which is used where 
- * <function>poll()</function> isn't available).
+ * poll() isn't available).
  *
  * This function could possibly be used to integrate the GLib event
  * loop with an external event loop.
@@ -2863,8 +3008,8 @@ g_main_context_wakeup_unlocked (GMainContext *context)
  * g_main_context_wakeup:
  * @context: a #GMainContext
  * 
- * If @context is currently waiting in a <function>poll()</function>, interrupt
- * the <function>poll()</function>, and continue the iteration process.
+ * If @context is currently waiting in a poll(), interrupt
+ * the poll(), and continue the iteration process.
  **/
 void
 g_main_context_wakeup (GMainContext *context)
@@ -2937,11 +3082,11 @@ g_timeout_prepare  (GSource  *source,
 	   * this at least avoids hanging for long periods of time.
 	   */
 	  g_timeout_set_expiration (timeout_source, &current_time);
-	  msec = timeout_source->interval;
+	  msec = MIN (G_MAXINT, timeout_source->interval);
 	}
       else
 	{
-	  msec += sec * 1000;
+	  msec = MIN (G_MAXINT, (guint)msec + 1000 * (guint)sec);
 	}
     }
 
@@ -3076,8 +3221,7 @@ g_timeout_add_full (gint           priority,
  * Sets a function to be called at regular intervals, with the default
  * priority, #G_PRIORITY_DEFAULT.  The function is called repeatedly
  * until it returns %FALSE, at which point the timeout is automatically
- * destroyed and the function will not be called again.  The @notify
- * function is called when the timeout is destroyed.  The first call
+ * destroyed and the function will not be called again.  The first call
  * to the function will be at the end of the first @interval.
  *
  * Note that timeout functions may be delayed, due to the processing of other
@@ -3136,14 +3280,21 @@ g_idle_dispatch (GSource    *source,
  *
  * The source will not initially be associated with any #GMainContext
  * and must be added to one with g_source_attach() before it will be
- * executed.
+ * executed. Note that the default priority for idle sources is
+ * %G_PRIORITY_DEFAULT_IDLE, as compared to other sources which
+ * have a default priority of %G_PRIORITY_DEFAULT.
  * 
  * Return value: the newly-created idle source
  **/
 GSource *
 g_idle_source_new (void)
 {
-  return g_source_new (&g_idle_funcs, sizeof (GSource));
+  GSource *source;
+
+  source = g_source_new (&g_idle_funcs, sizeof (GSource));
+  g_source_set_priority (source, G_PRIORITY_DEFAULT_IDLE);
+
+  return source;
 }
 
 /**
@@ -3173,7 +3324,7 @@ g_idle_add_full (gint           priority,
 
   source = g_idle_source_new ();
 
-  if (priority != G_PRIORITY_DEFAULT)
+  if (priority != G_PRIORITY_DEFAULT_IDLE)
     g_source_set_priority (source, priority);
 
   g_source_set_callback (source, function, data, notify);
