@@ -30,6 +30,7 @@
 #include <stdio.h>
 #include <string.h>
 #include "glib.h"
+#include "glib/gprintf.h"
 
 int array[10000];
 gboolean failed = FALSE;
@@ -107,19 +108,19 @@ main (int   argc,
 #ifndef G_OS_WIN32
   /* MSVC and mingw32 use the same run-time C library, which doesn't like
      the %10000.10000f format... */
-  g_string_sprintf (string2, "%s|%0100d|%s|%s|%0*d|%*.*f|%10000.10000f",
-		    "this pete guy sure is a wuss, like he's the number ",
-		    1,
-		    " wuss.  everyone agrees.\n",
-		    string1->str,
-		    10, 666, 15, 15, 666.666666666, 666.666666666);
+  g_string_printf (string2, "%s|%0100d|%s|%s|%0*d|%*.*f|%10000.10000f",
+		   "this pete guy sure is a wuss, like he's the number ",
+		   1,
+		   " wuss.  everyone agrees.\n",
+		   string1->str,
+		   10, 666, 15, 15, 666.666666666, 666.666666666);
 #else
-  g_string_sprintf (string2, "%s|%0100d|%s|%s|%0*d|%*.*f|%100.100f",
-		    "this pete guy sure is a wuss, like he's the number ",
-		    1,
-		    " wuss.  everyone agrees.\n",
-		    string1->str,
-		    10, 666, 15, 15, 666.666666666, 666.666666666);
+  g_string_printf (string2, "%s|%0100d|%s|%s|%0*d|%*.*f|%100.100f",
+		   "this pete guy sure is a wuss, like he's the number ",
+		   1,
+		   " wuss.  everyone agrees.\n",
+		   string1->str,
+		   10, 666, 15, 15, 666.666666666, 666.666666666);
 #endif
   
   g_string_free (string1, TRUE);
@@ -207,9 +208,33 @@ main (int   argc,
   g_assert ( memcmp(string1->str, "fiddle\0xyzzy", 13) == 0);
   g_string_insert(string1, 1, "QED");
   g_assert ( memcmp(string1->str, "fQEDiddle\0xyzzy", 16) == 0);
+  g_string_printf (string1, "fiddle%cxyzzy", '\0');
+  g_assert (string1->len == 12);
+  g_assert (memcmp (string1->str, "fiddle\0xyzzy", 13) == 0);
+
   g_string_free (string1, TRUE);
   g_string_free (string2, TRUE);
   
+  g_assert (g_str_has_prefix("foobar", "gazonk") == FALSE);
+  g_assert (g_str_has_prefix("xyzzy", "xyzzy") == TRUE);
+  g_assert (g_str_has_prefix("xyzzy", "xy") == TRUE);
+  g_assert (g_str_has_prefix("xyzzy", "") == TRUE);
+  g_assert (g_str_has_prefix("xyz", "xyzzy") == FALSE);
+  g_assert (g_str_has_prefix("", "xyzzy") == FALSE);
+  g_assert (g_str_has_prefix("", "") == TRUE);
+
+  g_assert (g_str_has_suffix("foobar", "gazonk") == FALSE);
+  g_assert (g_str_has_suffix("xyzzy", "xyzzy") == TRUE);
+  g_assert (g_str_has_suffix("xyzzy", "zy") == TRUE);
+  g_assert (g_str_has_suffix("xyzzy", "") == TRUE);
+  g_assert (g_str_has_suffix("zzy", "xyzzy") == FALSE);
+  g_assert (g_str_has_suffix("", "xyzzy") == FALSE);
+  g_assert (g_str_has_suffix("", "") == TRUE);
+
+  tmp_string = (gchar *) g_malloc (10);
+  g_snprintf (tmp_string, 10, "%2$s %1$s", "a", "b");
+  g_assert (strcmp (tmp_string, "b a") == 0);
   return 0;
 }
 
+
