@@ -307,37 +307,20 @@ my_traverse (gpointer key,
   return FALSE;
 }
 
-static gboolean 
-find_first_that(gpointer key, 
-		gpointer value, 
-		gpointer user_data)
-{
-  gint *v = value;
-  gint *test = user_data;
-  return (*v == *test);
-}
-
-
 int
 main (int   argc,
       char *argv[])
 {
-  const gchar *s;
-  gchar **sv;
   GList *list, *t;
   GSList *slist, *st;
   GHashTable *hash_table;
   GMemChunk *mem_chunk;
   GStringChunk *string_chunk;
-  GTimer *timer, *timer2;
-  gdouble elapsed;
-  gulong elapsed_usecs;
+  GTimer *timer;
   gint nums[10] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
   gint morenums[10] = { 8, 9, 7, 0, 3, 2, 5, 1, 4, 6};
   gchar *string;
-  gint value = 120; 
-  gint *pvalue=NULL; 
-  
+
   gchar *mem[10000], *tmp_string = NULL, *tmp_string_2;
   gint i, j;
   GArray *garray;
@@ -438,31 +421,18 @@ main (int   argc,
   g_free (string);
   g_print ("user: %s\n", g_get_user_name ());
   g_print ("real: %s\n", g_get_real_name ());
-  s = g_get_home_dir ();
-  g_print ("home: %s\n", s ? s : "NULL!");
-  s = g_get_user_data_dir ();
-  g_print ("user_data: %s\n", s ? s : "NULL!");
-  s = g_get_user_config_dir ();
-  g_print ("user_config: %s\n", s ? s : "NULL!");
-  s = g_get_user_cache_dir ();
-  g_print ("user_cache: %s\n", s ? s : "NULL!");
-  sv = (gchar **) g_get_system_data_dirs ();
-  g_print ("system_data: %s\n", sv ? g_strjoinv (G_SEARCHPATH_SEPARATOR_S, sv) : "NULL!");
-  sv = (gchar **) g_get_system_config_dirs ();
-  g_print ("system_config: %s\n", s ? g_strjoinv (G_SEARCHPATH_SEPARATOR_S, sv) : "NULL!");
+  g_print ("home: %s\n", g_get_home_dir ());
   g_print ("tmp-dir: %s\n", g_get_tmp_dir ());
-  sv = (gchar **) g_get_language_names ();
-  g_print ("languages: %s\n", s ? g_strjoinv (":", sv) : "NULL!");
 
   /* type sizes */
-  g_print ("checking size of gint8: %"    G_GSIZE_FORMAT, sizeof (gint8));
+  g_print ("checking size of gint8: %d", (int)sizeof (gint8));
   TEST (NULL, sizeof (gint8) == 1);
-  g_print ("\nchecking size of gint16: %" G_GSIZE_FORMAT, sizeof (gint16));
+  g_print ("\nchecking size of gint16: %d", (int)sizeof (gint16));
   TEST (NULL, sizeof (gint16) == 2);
-  g_print ("\nchecking size of gint32: %" G_GSIZE_FORMAT, sizeof (gint32));
+  g_print ("\nchecking size of gint32: %d", (int)sizeof (gint32));
   TEST (NULL, sizeof (gint32) == 4);
-  g_print ("\nchecking size of gsize: %"  G_GSIZE_FORMAT, sizeof (gsize));
-  g_print ("\nchecking size of gint64: %" G_GSIZE_FORMAT, sizeof (gint64));
+  g_print ("\nchecking size of gsize: %d", (int)sizeof (gsize));
+  g_print ("\nchecking size of gint64: %d", (int)sizeof (gint64));
   TEST (NULL, sizeof (gint64) == 8);
   g_print ("\n");
 
@@ -750,10 +720,6 @@ main (int   argc,
       array[i] = i;
       g_hash_table_insert (hash_table, &array[i], &array[i]);
     }
-  pvalue = g_hash_table_find (hash_table, find_first_that, &value);
-  if (*pvalue != value)
-	  g_print("g_hash_table_find failed");
-  
   g_hash_table_foreach (hash_table, my_hash_callback, NULL);
 
   for (i = 0; i < 10000; i++)
@@ -980,60 +946,6 @@ main (int   argc,
 
   g_print ("ok\n");
 
-  g_print ("checking g_timer_continue...\n");
-
-  timer2 = g_timer_new ();
-
-  g_print ("\trun for 1 second...\n");
-  timer = g_timer_new();
-  g_usleep(G_USEC_PER_SEC); /* run timer for 1 second */
-  g_timer_stop(timer);
-
-  g_print ("\tstop for 1 second...\n");
-  g_usleep(G_USEC_PER_SEC); /* wait for 1 second */
-  g_print ("\trun for 2 seconds...\n");
-
-  g_timer_continue(timer);
-  g_usleep(2*G_USEC_PER_SEC); /* run timer for 2 seconds */
-  g_timer_stop(timer);
-
-  g_print ("\tstop for 1.5 seconds...\n");
-  g_usleep((3*G_USEC_PER_SEC)/2); /* wait for 1.5 seconds */
-  g_print ("\trun for 0.2 seconds...\n");
-
-  g_timer_continue(timer);
-  g_usleep(G_USEC_PER_SEC/5); /* run timer for 0.2 seconds */
-  g_timer_stop(timer);
-
-  g_print ("\tstop for 4 seconds...\n");
-  g_usleep(4*G_USEC_PER_SEC); /* wait for 4 seconds */
-  g_print ("\trun for 5.8 seconds...\n");
-
-  g_timer_continue(timer);
-  g_usleep((29*G_USEC_PER_SEC)/5); /* run timer for 5.8 seconds */
-  g_timer_stop(timer);
-
-  elapsed = g_timer_elapsed (timer, &elapsed_usecs);
-  g_print ("\t=> timer = %.6f = %d.%06ld (should be: 9.000000) (%.6f off)\n", elapsed, (int) elapsed, elapsed_usecs, ABS (elapsed - 9.));
-
-  if (elapsed > 8.8 && elapsed < 9.2)
-    g_print ("g_timer_continue ... ok\n\n");
-  else
-    g_print ("g_timer_continue ... ***** FAILED *****\n\n");
-
-  g_timer_stop(timer2);
-
-  elapsed = g_timer_elapsed(timer2, &elapsed_usecs);
-  g_print ("\t=> timer2 = %.6f = %d.%06ld (should be: %.6f) (%.6f off)\n\n", elapsed, (int) elapsed, elapsed_usecs, 9.+6.5, ABS (elapsed - (9.+6.5)));
-
-  if (elapsed > (8.8+6.5) && elapsed < (9.2+6.5))
-    g_print ("timer2 ... ok\n\n");
-  else
-    g_print ("timer2 ... ***** FAILED *****\n\n");
-
-  g_timer_destroy(timer);
-  g_timer_destroy(timer2);
-
   g_print ("checking g_ascii_strcasecmp...");
   g_assert (g_ascii_strcasecmp ("FroboZZ", "frobozz") == 0);
   g_assert (g_ascii_strcasecmp ("frobozz", "frobozz") == 0);
@@ -1251,8 +1163,6 @@ main (int   argc,
   g_message ("the next warning is a test:");
   string = NULL;
   g_print (string);
-  g_message ("non-printable UTF-8: \"\xc3\xa4\xda\x85\"");
-  g_message ("unsafe chars: \"\x10\x11\x12\n\t\x7f\x81\x82\x83\"");
 
   g_print ("checking endian macros (host is ");
 #if G_BYTE_ORDER == G_BIG_ENDIAN
@@ -1266,7 +1176,7 @@ main (int   argc,
 
   g_print ("ok\n");
 
-  if (g_get_charset ((G_CONST_RETURN char**)&charset))
+  if (g_get_charset (&charset))
     g_print ("current charset is UTF-8: %s\n", charset);
   else
     g_print ("current charset is not UTF-8: %s\n", charset);
@@ -1290,8 +1200,6 @@ main (int   argc,
   g_print ("found more.com as %s\n", g_find_program_in_path ("more.com"));
   g_print ("found regedit as %s\n", g_find_program_in_path ("regedit"));
 
-  g_print ("a Win32 error message: %s\n", g_win32_error_message (2));
-
 #endif
 
   g_print ("checking file functions...\n");
